;------------------------------------------------------------
; File Name :      pic18.inc
;------------------------------------------------------------
;      Copyright: Eugene M. Hutorny  2006
;      Revision:  V1.0
;      Date:      Nov 19, 2006
;      License:   Licensed to public under the Open Software License 
;                 http://www.opensource.org/licenses/osl-3.0.php
;------------------------------------------------------------
;
; ----------------------------------------------------------
; Family 18 specific primitives
; ----------------------------------------------------------
           SUBTITLE        "Family 18 specific definitions"
; ----------------------------------------------------------
; Should be included from pic.inc only
_pic_fam        set _pic18          ; set PIC18 family
_pic_step       equ 2               ;
_pic_bankmask   equ 0FFh
_pic_shrram     equ 00h             ; no shared memory
; ----------------------------------------------------------

ei macro
    bsf  INTCON, GIE
  endm

di macro
	bcf  INTCON, GIE
  endm

loadf macro dst, literal   
    movlw   literal
    movwf   dst
  endm

loadf16 macro dst, literal
    movlw   low(literal)
    movwf   dst   
    movlw   high(literal)
    movwf   dst+1
 endm

; ----------------------------------------------------------
; INDF/FSR primitives
;
; store file into indirect location, increment FSR
INDF_put macro file			; *FSR++ = file
	movff  file, INDF
	incf   FSR, F
  endm

; load FSR with a literal pointing to a constant location
FSR_setl macro literal         ; FSR = const
    bankisel literal
	movlw  literal
	movwf  FSR
  endm  

; load FSR with a file pointing to a location
FSR_setf macro file        ; FSR = file
    movf   file, W         
    movwf  FSR
  endm

; load FSR with a file pointing to a location, set bankisel  
FSR_setfl macro file, literal       ; FSR = file
    bankisel literal
    movf   file, W         
    movwf  FSR
  endm

; ----------------------------------------------------------
PCLATH_set macro lbl
	movlw	upper(lbl)
	movwf	PCLATU
	movlw	high(lbl)
	movwf	PCLATH
 endm

; ----------------------------------------------------------
; Ignore _pic_banksel
_pic_bankinit macro
 endm
_pic_banksel macro SFR
 endm

; ----------------------------------------------------------
; PIC16-PIC18 compatibility layer

#define udata_shr udata_acs

skpnz macro
    btfsc STATUS,Z
 endm

skpz macro
    btfss STATUS,Z
 endm

skpnc macro
    btfsc STATUS,C
 endm

skpc macro
    btfss STATUS,C
 endm

skpndc macro
    btfsc STATUS,DC
 endm

rlf macro file, d
    rlcf file, d
 endm

rrf macro file, d
    rrcf file, d
 endm

movfw macro file
    movf    file, W
 endm

tstf macro file
    movf    file, F
  endm

setc macro
    bsf STATUS,C
  endm

clrc macro
    bcf STATUS,C
  endm

setz macro
    bsf STATUS,Z
  endm

clrz macro
    bcf STATUS,Z
  endm

clrw macro
    clrf WREG
  endm

FSR             equ FSR0L
INDF            equ INDF0
SSPCON          equ SSPCON1

#define bankisel FSR0_bankisel

FSR0_bankisel macro label
    ;messg "FSR0L is overwritten"
    lfsr   FSR0, label
  endm

; Ignore pagesel
#define pagesel pagesel_ignore

pagesel_ignore macro label
 endm

_sfr_banksel macro sfr
 endm

#define banksel BSR_banksel
BSR_banksel macro label
    movlb   high(label)
 endm

; ----------------------------------------------------------
pic_mPortToTris macro ; pic_mPortToTris(port * FSR)
    movlw       TRISA - PORTA
    btfsc       FSR, .3         ; if FSR points to LAT* 
    movlw       TRISA - LATA
    addwf       FSR, F          ; FSR = FSR + TRISA - PORTA
 endm

; ----------------------------------------------------------
; Xor based switch/case
; Usage:
; swicth file               ; 1 instruction
; case const1, label1       ; 3 instructions
; case const2, label2       ; 3 instructions

 variable switch_val
switch macro file
switch_val = 0
    movfw   file
 endm

case macro const, label
    xorlw  const ^ switch_val
    bz label
switch_val = const
 endm
